% matlab code for LAS
clear;clc;
%% load the data
filepath = '.';
dataFilename = 'breast_Matlab.txt';

threads = 1;
BCnumPositive = 10;
BCnumNegative = 10;
iterationsPerBC = 1000;
scoreThreshold = 1;
%%
data = U_caching_txt_file_load(fullfile(filepath,dataFilename));
% data = uiimport('-file');
%% preprocessing

% Column Center
CC = @(x)(x-repmat(mean(x,1),[size(x,1) 1]));

% Column Standardize
CS_ = @(x)(x./repmat(std(x),size(x,1),1));
CS = @(x)CS_(CC(x));

%  The bend tails LN transform
LN_ = @(x)(sign(x).*log(1+abs(x)));
LN = @(x)CS(LN_(x));

data = CS(data);
dataLN = LN(data);

dataCur = mean(data(:).^4);
dataLNcur = mean(dataLN(:).^4);

if(abs(dataLNcur-3) < abs(dataCur-3))
	data = dataLN;
end;

clear dataLN;
clear RMC CC CS CS_ LN LN_ dataCur dataLNcur;
%% 
if(threads>1)
	if(matlabpool('size')==0)
		matlabpool 4
	end;
end;
RedBiclusters = LAS_SearchForRedBCs( data,BCnumPositive,iterationsPerBC,scoreThreshold);
GrnBiclusters = LAS_SearchForRedBCs(-data,BCnumNegative,iterationsPerBC,scoreThreshold);
if(matlabpool('size')>0)
    matlabpool close
end;

%% Save the results

rowSetsCell = cell(length(RedBiclusters) + length(GrnBiclusters),1);
colSetsCell = cell(length(RedBiclusters) + length(GrnBiclusters),1);
labels = cell(length(RedBiclusters) + length(GrnBiclusters),1);

for i=1:length(RedBiclusters)
	rowSetsCell{i} = RedBiclusters(i).rows';
	colSetsCell{i} = RedBiclusters(i).cols;
	labels{i} = ...
		['Red_' num2str(i) '\tbicluster of\t' ...
		num2str(sum(RedBiclusters(i).rows))	'x'	num2str(sum(RedBiclusters(i).cols))	'\tsize with average\t' ...
		num2str(RedBiclusters(i).avg) '\tand score\t' num2str(RedBiclusters(i).score) '\n'];
end;
for i=1:length(GrnBiclusters)
	rowSetsCell{i+length(RedBiclusters)} = GrnBiclusters(i).rows';
	colSetsCell{i+length(RedBiclusters)} = GrnBiclusters(i).cols;
	labels{i+length(RedBiclusters)} = ...
		['Green_' num2str(i) '\tbicluster of\t' ...
		num2str(sum(RedBiclusters(i).rows))	'x'	num2str(sum(RedBiclusters(i).cols))	'\tsize with average\t' ...
		num2str(-RedBiclusters(i).avg) '\tand score\t' num2str(RedBiclusters(i).score) '\n'];
end;

rowSets = cat(1,rowSetsCell{:});
colSets = cat(1,colSetsCell{:});

U_SaveIntegerText([filepath dataFilename '.Rows.binary.txt'],rowSets);
U_SaveIntegerText([filepath dataFilename '.Columns.binary.txt'],colSets);

fid = fopen( [filepath dataFilename '.Labels.txt'],'wt');
for i = 1:size(labels,1)
	fprintf(fid,labels{i});
end;
fclose(fid);

save([filepath dataFilename '.biclusters.mat'],'RedBiclusters','GrnBiclusters');




























